
#include "stacks.ch"

STATIC aSETStack  := {} // contains stacked SET() values
STATIC aAreaSetup := {} // contains stack of information saved
STATIC aDBFSetup  := {} // contain DBF stacked info
STATIC aRELSetup  := {} // contains subarrays, second item in subarray
                        // is another array containing the relations themselves

//===========================================================================
PROCEDURE PushSet(nValue, xNewVal)
//---------------------------------------------------------------------------
// PURPOSE: to store and optionally reset a value from a SET()
//
// PARAMETERS: nValue -> any value valid for SET()
//             xNewVal -> optional, new value for that specified SET()
//---------------------------------------------------------------------------

  IF (PCOUNT()==1)
    AADD(aSETStack, {nValue, SET(nValue)} )
  ELSE
    AADD(aSETStack, {nValue, SET(nValue, xNewVal)} )
  ENDIF

RETURN


//===========================================================================
PROCEDURE PopSet()
//---------------------------------------------------------------------------
// PURPOSE: to restore the last saved setting
//---------------------------------------------------------------------------
  LOCAL aSetting

  aSetting := ATAIL(aSETStack)

  SET(aSetting[SYSTEM_SETTING], aSetting[SYSTEM_VALUE])

  ASIZE(aSETStack, LEN(aSETStack)-1)    // kill end of queue

RETURN

//===========================================================================
PROCEDURE PushDBF(cDBF)
//---------------------------------------------------------------------------
// PURPOSE: to save databases info
// NOTE: assumes that the database is already openned
//---------------------------------------------------------------------------
  LOCAL aThisDBF := ARRAY(DBF_LENGTH)

  DEFAULT cDBF TO ALIAS() // if not DBF given, default to current

  aThisDBF[DBF_SELECT] := SELECT()   // selected area

  DBSELECTAREA(cDBF)
  aThisDBF[DBF_DBF]    := SELECT()   // new area
  aThisDBF[DBF_ORDER]  := INDEXORD() // last record position
  aThisDBF[DBF_RECNO]  := RECNO()    // last order position

  AADD(aDBFSetup, aThisDBF) // add it to stack

RETURN


//===========================================================================
PROCEDURE PopDBF()
//---------------------------------------------------------------------------
// PURPOSE: to restore information about the DBF
//---------------------------------------------------------------------------
  LOCAL aThisDBF

  aThisDBF := ATAIL(aDBFSetup)

  DBSELECTAREA(aThisDBF[DBF_DBF])       // re-select new (in case not current)
  IF !EMPTY(ALIAS())
    DBSETORDER(aThisDBF[DBF_ORDER])     // reset values for new
    DBGOTO(aThisDBF[DBF_RECNO])
  ENDIF

  DBSELECTAREA(aThisDBF[DBF_SELECT])    // reselect old

  ASIZE(aDBFSetup, LEN(aDBFSetup)-1)    // kill end of queue

RETURN


//===========================================================================
PROCEDURE PushRelation(cKey, cDBF)
//---------------------------------------------------------------------------
// PURPOSE: to save the relationships for the current database
//
// NOTE: format of the aRELSetup array
//
//  aRELSetup[n] -> { {KEY, DBF}, ... }
//
// I.E.  aRELSetup[n] -> array of relations information
//
//       aRELSetup[n][n] -> relation information for each relations
//       aRELSetup[n][n][1] -> relation key (TO)
//       aRELSetup[n][n][2] -> relation dbf (INTO)
//
//---------------------------------------------------------------------------
  LOCAL n
  LOCAL aThis

  aThis := {}
  n := 1
  WHILE !EMPTY(DBRELATION(n))
    AADD(aThis, {DBRELATION(n), DBRSELECT(n)})
    n++
  END

  AADD(aRELSetup, aThis)  // add to aRELSetup array

  IF (cDBF != NIL)
    SET RELATION TO &cKey. INTO (cDBF)
  ENDIF

RETURN


//===========================================================================
PROCEDURE PopRelation()
//---------------------------------------------------------------------------
// PURPOSE: to reset the relationship as it was
//---------------------------------------------------------------------------
  LOCAL aThis := ATAIL(aRELSetup)
  LOCAL n
  LOCAL cKeyTo

  SET RELATION TO     // reset relations prior to re-adding

  IF !EMPTY(aThis)
    FOR n := 1 TO LEN(aThis)
      cKeyTo := aThis[n][REL_TO]
      SET RELATION TO &cKeyTo INTO (ALIAS(aThis[n][REL_INTO])) ADDITIVE
    NEXT
  ENDIF

  ASIZE(aRELSetup, LEN(aRELSetup)-1)

RETURN



